from langchain.chains import ConversationalRetrievalChain
from langchain_openai import ChatOpenAI
from utils.pdf_loader import load_pdfs
from langchain.prompts import PromptTemplate
from langchain.memory import ConversationBufferMemory

class ChatAgent:
    def __init__(self, pdf_dir):
        # 初始化 LLM
        self.llm = ChatOpenAI(
            model='deepseek-chat',
            openai_api_key='',
            openai_api_base='https://api.deepseek.com',
            max_tokens=1024,
            streaming=True
        )
        
        # 加载知识库
        self.vectorstore = load_pdfs(pdf_dir)
        
        # 创建自定义提示模板
        prompt_template = """基于以下已知信息，简洁和专业的来回答用户的问题。
        如果无法从中得到答案，请说 "抱歉，我无法从已知信息中找到答案。"
        
已知信息：
{context}

问题：{question}

回答："""
        
        # 优化检索设置
        retriever = self.vectorstore.as_retriever(
            search_type="similarity",
            search_kwargs={
                "k": 5,  # 增加返回的相关文档数量
            }
        )
        
        # 创建对话链
        self.chain = ConversationalRetrievalChain.from_llm(
            llm=self.llm,
            retriever=retriever,  # 使用优化后的检索器
            memory=ConversationBufferMemory(
                memory_key="chat_history",
                return_messages=True,
                output_key='answer'
            ),
            combine_docs_chain_kwargs={
                "prompt": PromptTemplate(
                    template=prompt_template,
                    input_variables=["context", "question"]
                )
            },
            return_source_documents=False,
            verbose=False
        )
    
    def chat(self, query):
        """处理用户输入并返回响应"""
        response = self.chain.stream({
            "question": query,
            "chat_history": []  # 提供空的聊天历史
        })
        # 只返回回答内容
        for chunk in response:
            if isinstance(chunk, str):
                yield chunk
            elif isinstance(chunk, dict) and 'answer' in chunk:
                yield chunk['answer']